<?php

declare(strict_types=1);

namespace Drupal\drupalorg_crosssite\EventSubscriber;

use Drupal\Core\Routing\CurrentRouteMatch;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Drupal.org cross-site functionality event subscriber.
 */
class CookieLoginSubscriber implements EventSubscriberInterface {

  /**
   * Route match service.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $currentRouteMatch;

  /**
   * The current account.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * Constructs a new CookieLoginSubscriber.
   *
   * @param \Drupal\Core\Routing\CurrentRouteMatch $current_route_match
   *   The current route.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   */
  public function __construct(CurrentRouteMatch $current_route_match, AccountInterface $account) {
    $this->currentRouteMatch = $current_route_match;
    $this->account = $account;
  }

  /**
   * Logic to run on login.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   Event object.
   */
  public function onLogin(ResponseEvent $event): void {
    if ($this->account->isAuthenticated() && in_array($this->currentRouteMatch->getRouteName(), [
      'user.reset.login',
      'user.login'
    ])) {
      $response = $event->getResponse();
      $request = $event->getRequest();

      // Logged-in users have consented to the ToS.
      if ($request->cookies->has('cookieConsent') && $request->cookies->get('cookieConsent') === 'optout') {
        $consent_cookie = new Cookie('cookieConsent', 'optin');
        $response->headers->setCookie($consent_cookie);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::RESPONSE => ['onLogin'],
    ];
  }

}
