<?php

declare(strict_types=1);

namespace Drupal\drupalorg_crosssite\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\drupalorg_crosssite\BaseFields;
use Drupal\field\Entity\FieldConfig;
use Drupal\file\FileRepositoryInterface;
use Drupal\social_auth\Event\LoginEvent;
use Drupal\social_auth\Event\SocialAuthEvents;
use Drupal\social_auth\User\SocialAuthUserInterface;
use Drupal\token\Token;
use Drupal\user\UserDataInterface;
use Drupal\user\UserInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\RequestOptions;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Drupal.org cross-site functionality event subscriber.
 */
final class KeycloakLoginSubscriber implements EventSubscriberInterface {

  /**
   * Constructor method.
   *
   * @readonly \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager instance.
   * @readonly \GuzzleHttp\ClientInterface $httpClient
   *   Http client.
   * @readonly \Drupal\Core\File\FileSystemInterface $fileSystem
   *   File system instance.
   * @readonly \Drupal\user\UserDataInterface $userData
   *   User data instance.
   * @readonly \Drupal\file\FileRepositoryInterface $fileRepository
   *   File repository instance.
   * @readonly \Drupal\token\Token $tokenizer
   *   Tokenizer service.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly ClientInterface $httpClient,
    private readonly FileSystemInterface $fileSystem,
    private readonly UserDataInterface $userData,
    private readonly FileRepositoryInterface $fileRepository,
    private readonly Token $tokenizer,
    private readonly ConfigFactoryInterface $config
  ) {}

  /**
   * Logic to run on login.
   *
   * @param \Drupal\social_auth\Event\LoginEvent $event
   *   Event object.
   */
  public function onLogin(LoginEvent $event): void {
    if ($this->config->get('drupalorg_crosssite.settings')->get('disable_sso')) {
      return;
    }

    $account = $event->getDrupalAccount();
    $social_auth_user = $event->getSocialAuthUser();
    $user = $this->entityTypeManager->getStorage('user')->load($account->id());
    assert($user instanceof UserInterface);
    $original = $user->toArray();
    $user->setUsername($social_auth_user->getName())
      ->setEmail($social_auth_user->getEmail())
      ->set(BaseFields::ORIGINAL_DRUPAL_ID, $social_auth_user->getData('origin_id'))
      ->set('timezone', $social_auth_user->getData('zoneinfo'));
    $this->setCreated($user, $social_auth_user);
    $this->setNames($user, $social_auth_user);
    $this->manageRoles($user, $social_auth_user);
    $this->saveUserPicture($user, $social_auth_user);
    if ($original !== $user->toArray()) {
      $user->save();
    }
  }

  /**
   * @param \Drupal\user\UserInterface $user
   *   The Drupal user.
   * @param \Drupal\social_auth\User\SocialAuthUserInterface $socialAuthUser
   *   The SocialAuth user.
   *
   * @return void
   */
  private function setNames(UserInterface $user, SocialAuthUserInterface $socialAuthUser): void {
    if ($user->hasField('field_crosssite_first_name')) {
      $user->set('field_crosssite_first_name', $socialAuthUser->getFirstName());
    }
    if ($user->hasField('field_crosssite_last_name')) {
      $user->set('field_crosssite_last_name', $socialAuthUser->getLastName());
    }
  }

  /**
   * Saves the user picture.
   *
   * @param \Drupal\user\UserInterface $user
   *   The Drupal user.
   * @param \Drupal\social_auth\User\SocialAuthUserInterface $socialAuthUser
   *   The SocialAuth user.
   */
  private function saveUserPicture(UserInterface $user, SocialAuthUserInterface $socialAuthUser): void {
    if (!$user->hasField('user_picture')) {
      return;
    }
    $user_photo_empty = $user->get('user_picture')->isEmpty();
    $user_photo_url = $socialAuthUser->getPictureUrl();
    // Return early if no user picture is available.
    if ($user_photo_url === NULL) {
      // And null out the picture if one previously existed.
      if (!$user_photo_empty) {
        $user->set('user_picture', NULL);
      }
      return;
    }

    $field_config = FieldConfig::loadByName('user', 'user', 'user_picture');
    $scheme = $field_config->getSetting('uri_scheme');
    $destination = $scheme . '://' . $this->tokenizer->replacePlain($field_config->getSetting('file_directory'));
    $result = $this->fileSystem->prepareDirectory($destination, FileSystemInterface::CREATE_DIRECTORY);
    if ($result === FALSE) {
      return;
    }
    $response = $this->httpClient->get($user_photo_url, [
      RequestOptions::HTTP_ERRORS => FALSE,
      RequestOptions::TIMEOUT => 3.0,
    ]);
    if ($response->getStatusCode() !== Response::HTTP_OK) {
      return;
    }
    $data = (string) $response->getBody();
    $new_hash = hash('crc32b', $data);
    $old_hash = $this->userData->get('drupalorg_crosssite', $user->id(), 'picture_hash');
    if ($new_hash === $old_hash) {
      return;
    }
    $extension = pathinfo($user_photo_url, PATHINFO_EXTENSION);
    $path = $user_photo_empty ?
      sprintf('%s/picture-%d.%s', $destination, $user->id(), $extension) :
      $user->get('user_picture')->entity->uri->value;
    $file = $this->fileRepository->writeData($data, $path, FileSystemInterface::EXISTS_REPLACE);
    $user->set('user_picture', $file->id());
    $this->userData->set('drupalorg_crosssite', $user->id(), 'picture_hash', $new_hash);
  }

  /**
   * Manages created timestamp.
   *
   * @param \Drupal\user\UserInterface $user
   *   The Drupal user.
   * @param \Drupal\social_auth\User\SocialAuthUserInterface $socialAuthUser
   *   The SocialAuth user.
   *
   * @return void
   */
  private function setCreated(UserInterface $user, SocialAuthUserInterface $socialAuthUser): void {
    $data = $socialAuthUser->getAdditionalData();
    // Keycloak didn't support importing a create timestamp. Here we calculate
    // if there is a legacy timestamp that is lower than Keycloak's timestamp.
    // @see https://github.com/keycloak/keycloak/issues/14009
    $legacy = $data['main']['legacy_created'] ?? -1;
    $created = $data['main']['created'];
    if ($legacy > 0 && $legacy < $created) {
      $created = $legacy;
    }
    // Keycloak stores time as milliseconds while Drupal in full seconds.
    // Calculate and round to the nearest second.
    $user->set('created', round($created * 0.001));
  }

  /**
   * Manages some roles for spam protection.
   *
   * @param \Drupal\user\UserInterface $user
   *   The Drupal user.
   * @param \Drupal\social_auth\User\SocialAuthUserInterface $socialAuthUser
   *   The SocialAuth user.
   *
   * @return void
   */
  private function manageRoles(UserInterface $user, SocialAuthUserInterface $socialAuthUser): void {
    if (!$this->config->get('drupalorg_crosssite.settings')->get('sub_site')) {
      return;
    }
    $data = $socialAuthUser->getAdditionalData();

    // Confirmed role.
    if (!empty($data['main']['groups'][0]) && in_array('/Confirmed', $data['main']['groups'], TRUE)) {
      $user->addRole('confirmed');
    }
    else {
      $user->removeRole('confirmed');
    }

    // Email verified.
    if (!empty($data['main']['email_verified']) && $data['main']['email_verified']) {
      $user->removeRole('email_unverified');
    }
    else {
      $user->addRole('email_unverified');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      SocialAuthEvents::USER_LOGIN => ['onLogin'],
    ];
  }

}
